@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_ui.c,v 1.15 2003/06/22 10:11:09 fuyu Exp $
+ * $Id: bbs_thread_ui.c,v 1.16 2003/07/13 14:53:51 fuyu Exp $
  */
 
 #include "config.h"
@@ -759,11 +759,16 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   OchushaAsyncBuffer *buffer = job_args->buffer;
   gchar default_text_buffer[DEFAULT_BUFFER_SIZE];
+  iconv_helper *helper = ochusha_bbs_thread_get_response_iconv_helper(thread);
   RendererContext context =
     {
       {
-	iconv_open("UTF-8//IGNORE",
-		   ochusha_bbs_thread_get_response_character_encoding(thread)),
+	helper != NULL
+	? iconv_open("UTF-8",
+		     ochusha_bbs_thread_get_response_character_encoding(thread))
+	: iconv_open("UTF-8//IGNORE",
+		     ochusha_bbs_thread_get_response_character_encoding(thread)),
+	helper,
 	start_element_cb,
 	end_element_cb,
 	NULL,
@@ -805,7 +810,7 @@ render_bbs_thread(WorkerThread *employee, gpointer args)
 
 #if DEBUG_GUI_MOST
   {
-    gchar *title = convert_string(utf8_to_native, thread->title, -1);
+    gchar *title = convert_string(utf8_to_native, NULL, thread->title, -1);
     fprintf(stderr, "render_bbs_thread: %s\n", title);
     G_FREE(title);
   }
@@ -977,8 +982,10 @@ render_response(OchushaBBSThread *thread, gint number,
   else if (strcmp(response->mailto, "sage") != 0)
     {
       gchar *name = simple_string_canon(response->name, -1,
-					context->handler.converter);
+					context->handler.converter,
+					context->handler.helper);
       gchar *mailto = convert_string(context->handler.converter,
+				     context->handler.helper,
 				     response->mailto, -1);
       bbs_thread_view_append_text_as_link(view, name, -1, mailto);
       G_FREE(mailto);
@@ -987,8 +994,10 @@ render_response(OchushaBBSThread *thread, gint number,
   else
     {
       gchar *name = simple_string_canon(response->name, -1,
-					context->handler.converter);
+					context->handler.converter,
+					context->handler.helper);
       gchar *mailto = convert_string(context->handler.converter,
+				     context->handler.helper,
 				     response->mailto, -1);
       bbs_thread_view_append_text_as_sage(view, name, -1, mailto);
       G_FREE(mailto);
@@ -1205,7 +1214,8 @@ start_element_cb(void *user_data, const char *name, const char *const attrs[])
 	if (strcasecmp(attrs[i * 2], "href") == 0)
 	  {
 	    link = convert_string(context->handler.converter,
-					 attrs[i * 2 + 1], -1);
+				  context->handler.helper,
+				  attrs[i * 2 + 1], -1);
 #if DEBUG_GUI_MOST
 	    fprintf(stderr, "href=\"%s\"\n", link);
 #endif
