@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.c,v 1.19 2003/06/07 14:35:30 fuyu Exp $
+ * $Id: bulletin_board_ui.c,v 1.25 2003/06/21 16:23:53 fuyu Exp $
  */
 
 #include "config.h"
@@ -59,7 +59,7 @@
 #include <string.h>
 
 
-static void open_thread_dat(WorkerThread *employee, gpointer job_args);
+static void open_thread_view(WorkerThread *employee, gpointer job_args);
 static void show_threadlist(WorkerThread *employee, gpointer args);
 static void advance_view_cb(ThreadlistView *view, OchushaBBSThread *thread,
 			    PanedNotebook *paned_notebook);
@@ -214,6 +214,8 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
   info->view_rank = ochusha_utils_get_attribute_int(hash_table, "view_rank");
   info->view_ignored = ochusha_utils_get_attribute_int(hash_table,
 						       "view_ignored");
+  if ((info->view_flags & BBS_THREAD_HIDDEN) != 0)
+    thread->number_of_responses_read = 0;
 
   return;
 }
@@ -235,7 +237,8 @@ threadlist_read_thread_element_cb(OchushaBulletinBoard *board,
 static void
 threadlist_write_thread_element_cb(OchushaBulletinBoard *board,
 				   OchushaBBSThread *thread,
-				   FILE *threadlist_xml, gpointer user_data)
+				   FILE *threadlist_xml,
+				   OchushaApplication *application)
 {
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
 
@@ -247,6 +250,12 @@ threadlist_write_thread_element_cb(OchushaBulletinBoard *board,
       OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info, view_rank);
       OUTPUT_THREAD_ATTRIBUTE_INT(threadlist_xml, info, view_ignored);
     }
+
+  if ((info->view_flags & BBS_THREAD_HIDDEN) != 0)
+    {
+      thread->number_of_responses_read = 0;
+      ochusha_bbs_thread_remove_cache(thread, &application->config);
+    }
 }
 
 
@@ -267,7 +276,8 @@ bulletin_board_gui_info_free(BulletinBoardGUIInfo *info)
 
 
 BulletinBoardGUIInfo *
-ensure_bulletin_board_info(OchushaBulletinBoard *board)
+ensure_bulletin_board_info(OchushaBulletinBoard *board,
+			   OchushaApplication *application)
 {
   BulletinBoardGUIInfo *info;
   THREAD_LIST_LOCK
@@ -284,7 +294,7 @@ ensure_bulletin_board_info(OchushaBulletinBoard *board)
 			   NULL);
 	  g_signal_connect(G_OBJECT(board), "threadlist_write_thread_element",
 			   G_CALLBACK(threadlist_write_thread_element_cb),
-			   NULL);
+			   application);
 	}
     }
   THREAD_LIST_UNLOCK;
@@ -343,6 +353,13 @@ title_match_func(const gchar *title, ThreadlistSearchQueryData *search_data)
 
   /* match_case˹碌normalizeƤ롣*/
   normalized_title = g_utf8_normalize(title, -1, G_NORMALIZE_ALL);
+#if TRACE_MEMORY_USAGE
+  {
+    gchar *tmp_title = normalized_title;
+    normalized_title = G_STRDUP(tmp_title);
+    g_free(tmp_title);
+  }
+#endif
   if (!search_data->match_case)
     {
       gchar *case_normalized_title = g_utf8_casefold(normalized_title, -1);
@@ -376,13 +393,64 @@ title_match_func(const gchar *title, ThreadlistSearchQueryData *search_data)
 }
 
 
+static void
+customize_threadlist_view_contents(OchushaApplication *application,
+				   ThreadlistView *view)
+{
+  const char *column = application->threadlist_view_contents;
+  if (column == NULL)
+    return;
+
+  g_return_if_fail(strlen(column) > 0);
+
+  while (*column != '\0')
+    {
+      switch (*column)
+	{
+	case 'R':
+	  threadlist_view_append_rank(view);
+	  break;
+
+	case 'M':
+	  threadlist_view_append_mark(view);
+	  break;
+
+	case 'T':
+	  threadlist_view_append_title(view);
+	  break;
+
+	case 'N':
+	  threadlist_view_append_number_of_responses(view);
+	  break;
+
+	case 'n':
+	  threadlist_view_append_number_of_responses_read(view);
+	  break;
+
+	case 'U':
+	  threadlist_view_append_number_of_responses_unread(view);
+	  break;
+
+	case 'D':
+	  threadlist_view_append_rank_difference(view);
+	  break;
+
+	default:
+	  fprintf(stderr, "'%c' doesn't match any column\n", *column);
+	  break;
+	}
+      column++;
+    }
+}
+
+
 GtkWidget *
 open_bulletin_board(OchushaApplication *application,
 		    OchushaBulletinBoard *board, IconLabel *tab_label)
 {
   GtkWidget *paned_notebook;
 
-  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
 
   if (info == NULL)
     {
@@ -407,6 +475,7 @@ open_bulletin_board(OchushaApplication *application,
 	}
 
       view = threadlist_view_new();
+      customize_threadlist_view_contents(application, THREADLIST_VIEW(view));
 
       job = G_NEW0(WorkerJob, 1);
       job_args = G_NEW0(ThreadlistJobArgs, 1);
@@ -508,18 +577,19 @@ open_bulletin_board(OchushaApplication *application,
       job->args = job_args;
 
       info->is_busy = TRUE;
-      info->subject_txt_buffer
+      info->threadlist_source_buffer
 	= ochusha_bulletin_board_get_threadlist_source(board,
 				application->broker,
 				OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH);
-      if (info->subject_txt_buffer != NULL)
-	setup_for_tab_label_animation(info->subject_txt_buffer, tab_label);
+      if (info->threadlist_source_buffer != NULL)
+	setup_for_tab_label_animation(info->threadlist_source_buffer,
+				      tab_label);
 
       /* Ȥޤǳ */
       g_object_ref(G_OBJECT(tab_label));
       g_object_ref(G_OBJECT(view));
-      if (info->subject_txt_buffer != NULL)
-	g_object_ref(G_OBJECT(info->subject_txt_buffer));
+      if (info->threadlist_source_buffer != NULL)
+	g_object_ref(G_OBJECT(info->threadlist_source_buffer));
 #if DEBUG_WIDGET
       fprintf(stderr, "threadlist_view->ref_count=%d\n",
 	      G_OBJECT(view)->ref_count);
@@ -539,7 +609,7 @@ typedef struct _OpenThreadJobArgs
   OchushaApplication *application;
   OchushaBulletinBoard *board;
   OchushaBBSThread *thread;
-  char *dat_filename;
+  char *thread_id;
   int res_num;
   gboolean in_tab;
 } OpenThreadJobArgs;
@@ -548,7 +618,7 @@ typedef struct _OpenThreadJobArgs
 void
 ochusha_open_thread(OchushaApplication *application,
 		    OchushaBulletinBoard *board, OchushaBBSThread *thread,
-		    const char *dat_filename, int res_num, gboolean in_tab)
+		    const char *thread_id, int res_num, gboolean in_tab)
 {
   WorkerJob *job;
   OpenThreadJobArgs *args;
@@ -567,13 +637,13 @@ ochusha_open_thread(OchushaApplication *application,
   args = G_NEW0(OpenThreadJobArgs, 1);
 
   job->canceled = FALSE;
-  job->job = open_thread_dat;
+  job->job = open_thread_view;
   job->args = args;
 
   args->application = application;
   args->board = board;
   args->thread = thread;
-  args->dat_filename = G_STRDUP(dat_filename);
+  args->thread_id = G_STRDUP(thread_id);
   args->res_num = res_num;
   args->in_tab = in_tab;
 
@@ -582,12 +652,12 @@ ochusha_open_thread(OchushaApplication *application,
 
 
 static void
-open_thread_dat(WorkerThread *employee, gpointer job_args)
+open_thread_view(WorkerThread *employee, gpointer job_args)
 {
   OpenThreadJobArgs *args = (OpenThreadJobArgs *)job_args;
   OchushaApplication *application = args->application;
   OchushaBulletinBoard *board = args->board;
-  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
 
   THREAD_LIST_LOCK
     {
@@ -597,10 +667,11 @@ open_thread_dat(WorkerThread *employee, gpointer job_args)
 	THREAD_LIST_COND_WAIT;
 
 #if DEBUG_GUI_MOST
-      fprintf(stderr, "DAT file: \"%s\"\n", args->dat_filename);
+      fprintf(stderr, "Thread ID (e.g. DAT file name): \"%s\"\n",
+	      args->thread_id);
 #endif
       /* λǡ٤Ϻ줿Ȥݾڤ */
-      thread = g_hash_table_lookup(board->thread_table, args->dat_filename);
+      thread = g_hash_table_lookup(board->thread_table, args->thread_id);
       gdk_threads_enter();
       board_view = (PanedNotebook *)paned_notebook_get_item_view(PANED_NOTEBOOK(application->contents_window), board);
 
@@ -616,7 +687,7 @@ open_thread_dat(WorkerThread *employee, gpointer job_args)
     }
   THREAD_LIST_UNLOCK;
 
-  G_FREE(args->dat_filename);
+  G_FREE(args->thread_id);
   G_FREE(args);
 }
 
@@ -631,7 +702,7 @@ refresh_threadlist_view(OchushaApplication *application,
   WorkerJob *job;
   ThreadlistJobArgs *job_args;
   IconLabel *tab_label;
-  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
 
   g_return_if_fail(board != NULL && application->contents_window != NULL);
   contents_window = PANED_NOTEBOOK(application->contents_window);
@@ -656,20 +727,20 @@ refresh_threadlist_view(OchushaApplication *application,
 	  return;
 	}
 
-      old_buffer = info->subject_txt_buffer;
+      old_buffer = info->threadlist_source_buffer;
 
-      info->subject_txt_buffer
+      info->threadlist_source_buffer
 	= ochusha_bulletin_board_get_threadlist_source(board,
 				application->broker,
 				OCHUSHA_NETWORK_BROKER_CACHE_TRY_REFRESH);
-      if (info->subject_txt_buffer == NULL)
+      if (info->threadlist_source_buffer == NULL)
 	{
-	  info->subject_txt_buffer = old_buffer;
+	  info->threadlist_source_buffer = old_buffer;
 	  THREAD_LIST_UNLOCK;
 	  return;
 	}
 
-      setup_for_tab_label_animation(info->subject_txt_buffer, tab_label);
+      setup_for_tab_label_animation(info->threadlist_source_buffer, tab_label);
 
       job = G_NEW0(WorkerJob, 1);
       job_args = G_NEW0(ThreadlistJobArgs, 1);
@@ -686,7 +757,7 @@ refresh_threadlist_view(OchushaApplication *application,
       
       info->is_busy = TRUE;
       /* λޤǳ */
-      g_object_ref(G_OBJECT(info->subject_txt_buffer));
+      g_object_ref(G_OBJECT(info->threadlist_source_buffer));
       g_object_ref(G_OBJECT(tab_label));
       g_object_ref(G_OBJECT(threadlist_view));
       commit_job(job);
@@ -707,7 +778,7 @@ redraw_threadlist_view(OchushaApplication *application,
   WorkerJob *job;
   ThreadlistJobArgs *job_args;
   IconLabel *tab_label;
-  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
 
   g_return_if_fail(board != NULL && application->contents_window != NULL);
   contents_window = PANED_NOTEBOOK(application->contents_window);
@@ -728,7 +799,7 @@ redraw_threadlist_view(OchushaApplication *application,
       while (info->is_busy)
 	THREAD_LIST_COND_WAIT;
 
-      if (info->subject_txt_buffer == NULL)
+      if (info->threadlist_source_buffer == NULL)
 	{
 	  THREAD_LIST_UNLOCK;
 	  return;
@@ -749,7 +820,7 @@ redraw_threadlist_view(OchushaApplication *application,
       
       info->is_busy = TRUE;
       /* λޤǳ */
-      g_object_ref(G_OBJECT(info->subject_txt_buffer));
+      g_object_ref(G_OBJECT(info->threadlist_source_buffer));
       g_object_ref(G_OBJECT(tab_label));
       g_object_ref(G_OBJECT(threadlist_view));
       commit_job(job);
@@ -803,17 +874,20 @@ each_thread_cb(OchushaBBSThread *thread, gpointer user_data)
 
   filter_result = threadlist_filter_check_thread(args->filter,
 						 args->application, thread);
-  if (filter_result || !args->enable_filter)
+  if (filter_result
+      || (!args->enable_filter
+	  && ((info->view_flags & BBS_THREAD_HIDDEN) == 0
+	      || !args->application->hide_hidden_threads)))
     {
       gdk_threads_enter();
 
 #if ENABLE_RENDERING_ON_IDLE
       while (!try_get_rendering_right(args->application,
-				      args->info->subject_txt_buffer,
+				      args->info->threadlist_source_buffer,
 				      thread->board))
 	{
 	  gdk_threads_leave();
-	  ochusha_async_buffer_wait(args->info->subject_txt_buffer,
+	  ochusha_async_buffer_wait(args->info->threadlist_source_buffer,
 				    "bulletin_board_ui.c: each_thread_cb()");
 	  gdk_threads_enter();
 	}
@@ -840,7 +914,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
   ThreadlistJobArgs *job_args = (ThreadlistJobArgs *)args;
   OchushaApplication *application = job_args->application;
   OchushaBulletinBoard *board = job_args->board;
-  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
   ThreadlistView *view = job_args->view;
   OchushaAsyncBuffer *buffer;
   EachThreadArgs cb_args =
@@ -867,7 +941,7 @@ show_threadlist(WorkerThread *employee, gpointer args)
 
   THREAD_LIST_LOCK
     {
-      buffer = info->subject_txt_buffer;
+      buffer = info->threadlist_source_buffer;
     }
   THREAD_LIST_UNLOCK;
 
@@ -942,16 +1016,17 @@ show_threadlist(WorkerThread *employee, gpointer args)
 
 
 OchushaAsyncBuffer *
-snatch_subject_txt_buffer_for_board(OchushaBulletinBoard *board)
+snatch_threadlist_source_buffer_for_board(OchushaBulletinBoard *board,
+					  OchushaApplication *application)
 {
-  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board);
+  BulletinBoardGUIInfo *info = ensure_bulletin_board_info(board, application);
   OchushaAsyncBuffer *buffer;
 
   THREAD_LIST_LOCK
     {
-      buffer = info->subject_txt_buffer;
+      buffer = info->threadlist_source_buffer;
       info->is_busy = FALSE;
-      info->subject_txt_buffer = NULL;
+      info->threadlist_source_buffer = NULL;
       THREAD_LIST_COND_BROADCAST;
     }
   THREAD_LIST_UNLOCK;
@@ -963,7 +1038,8 @@ void
 update_threadlist_entry_style(OchushaApplication *application,
 			      ThreadlistView *view, OchushaBBSThread *thread)
 {
-  BulletinBoardGUIInfo *board_info = ensure_bulletin_board_info(thread->board);
+  BulletinBoardGUIInfo *board_info = ensure_bulletin_board_info(thread->board,
+								application);
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
   ThreadlistEntryVisualParameter visual_parameter =
     {
@@ -1179,20 +1255,21 @@ item_view_required_cb(PanedNotebook *paned_notebook,
 
 
 static void
-free_dat_buffer(OchushaBBSThread *thread, gpointer unused)
+free_response_source_buffer(OchushaBBSThread *thread, gpointer unused)
 {
   BBSThreadGUIInfo *info = ensure_bbs_thread_info(thread);
 
 #if DEBUG_ASYNC_BUFFER_MOST
-  fprintf(stderr, "free_dat_buffer: unref OchushaAsyncBuffer(%p)\n",
-	  info->dat_buffer);
-  fprintf(stderr, "* before unref: dat_buffer->ref_count=%d\n",
-	  G_OBJECT(info->dat_buffer)->ref_count);
+  fprintf(stderr,
+	  "free_response_source_buffer: unref OchushaAsyncBuffer(%p)\n",
+	  info->response_source_buffer);
+  fprintf(stderr, "* before unref: response_source_buffer->ref_count=%d\n",
+	  G_OBJECT(info->response_source_buffer)->ref_count);
 #endif
-  if (info->dat_buffer != NULL)
+  if (info->response_source_buffer != NULL)
     {
-      g_object_unref(G_OBJECT(info->dat_buffer));
-      info->dat_buffer = NULL;
+      g_object_unref(G_OBJECT(info->response_source_buffer));
+      info->response_source_buffer = NULL;
     }
 }
 
@@ -1208,7 +1285,7 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook, OchushaBBSThread *threa
   g_return_if_fail(GTK_IS_WIDGET(item_view));
 
   info = ensure_bbs_thread_info(thread);
-  buffer = info->dat_buffer;
+  buffer = info->response_source_buffer;
 
   /*
    * XXX: Ǥinfo->bufferοǤthread safetyݤǤƤʤ
@@ -1263,8 +1340,8 @@ item_view_being_closed_cb(PanedNotebook *paned_notebook, OchushaBBSThread *threa
 
   ochusha_async_buffer_suspend_all();
 
-  if (info->dat_buffer != NULL)
-    ochusha_async_buffer_terminate(info->dat_buffer);
+  if (info->response_source_buffer != NULL)
+    ochusha_async_buffer_terminate(info->response_source_buffer);
 
   gdk_threads_enter();
 
@@ -1295,7 +1372,7 @@ item_view_closed_cb(PanedNotebook *paned_notebook, OchushaBBSThread *thread,
 
   info->last_read = NULL;
   info->next_mark = NULL;
-  free_dat_buffer(thread, NULL);
+  free_response_source_buffer(thread, NULL);
 }
 
 
@@ -1395,7 +1472,8 @@ finalize_bulletin_board_view(OchushaBulletinBoard *board, GtkWidget *widget)
     return;
 
   board_view = PANED_NOTEBOOK(widget);
-  paned_notebook_foreach_item(board_view, (GFunc)free_dat_buffer, NULL);
+  paned_notebook_foreach_item(board_view, (GFunc)free_response_source_buffer,
+			      NULL);
 #if DEBUG_WIDGET
   {
     ThreadlistView *threadlist_view
