@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bulletin_board_ui.h,v 1.6 2003/05/25 08:47:13 fuyu Exp $
+ * $Id: bulletin_board_ui.h,v 1.9 2003/06/20 18:14:40 fuyu Exp $
  */
 
 #ifndef _BULLETIN_BOARD_UI_H_
@@ -46,10 +46,9 @@ extern "C" {
 #endif
 
 
-/* BulletinBoardInfoΥ֥饹Ū˻Ȥ */
 typedef struct _BulletinBoardGUIInfo
 {
-  OchushaAsyncBuffer *subject_txt_buffer;
+  OchushaAsyncBuffer *threadlist_source_buffer;
 
   GtkWidget *properties_dialog;
 
@@ -80,7 +79,8 @@ typedef struct _ThreadlistSearchQueryData
 void prepare_board_ui_initialization(OchushaApplication *application);
 void initialize_board_ui(OchushaApplication *application);
 
-BulletinBoardGUIInfo *ensure_bulletin_board_info(OchushaBulletinBoard *board);
+BulletinBoardGUIInfo *ensure_bulletin_board_info(OchushaBulletinBoard *board,
+					OchushaApplication *application);
 GtkWidget *open_bulletin_board(OchushaApplication *application,
 			       OchushaBulletinBoard *board,
 			       IconLabel *tab_label);
@@ -89,14 +89,16 @@ void finalize_bulletin_board_view(OchushaBulletinBoard *board,
 
 void ochusha_open_thread(OchushaApplication *application,
 			 OchushaBulletinBoard *board, OchushaBBSThread *thread,
-			 const char *dat_filename, int res_num,
+			 const char *thread_id, int res_num,
 			 gboolean in_tab);
 
 void refresh_threadlist_view(OchushaApplication *application,
 			     OchushaBulletinBoard *board);
 void redraw_threadlist_view(OchushaApplication *application,
 			    OchushaBulletinBoard *board);
-OchushaAsyncBuffer *snatch_subject_txt_buffer_for_board(OchushaBulletinBoard *board);
+OchushaAsyncBuffer *snatch_threadlist_source_buffer_for_board(
+					OchushaBulletinBoard *board,
+					OchushaApplication *application);
 void update_threadlist_entry_style(OchushaApplication *application,
 				   ThreadlistView *view,
 				   OchushaBBSThread *thread);
