@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_ui.c,v 1.18 2003/06/21 16:23:53 fuyu Exp $
+ * $Id: ochusha_ui.c,v 1.19 2003/06/22 14:39:36 fuyu Exp $
  */
 
 #include "config.h"
@@ -1207,7 +1207,11 @@ access_failed_cb(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer,
   IconLabel *tab_label = g_object_get_qdata(G_OBJECT(buffer), tab_label_id);
 
 #if DEBUG_GUI
-  fprintf(stderr, "access_failed_cb\n");
+  gchar *native_reason = g_locale_from_utf8(reason, -1, NULL, NULL, NULL);
+  fprintf(stderr,
+	  "access_failed_cb(): buffer=%p, reason_code=%d, reason=\"%s\"\n",
+	  buffer, reason_code, native_reason);
+  g_free(native_reason);
 #endif
 
   if (tab_label == NULL)
