@@ -23,12 +23,15 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_filter.c,v 1.3 2003/05/15 19:55:43 fuyu Exp $
+ * $Id: threadlist_filter.c,v 1.6 2003/06/07 14:35:31 fuyu Exp $
  */
 
-#include "ochusha.h"
-#include "ochusha_ui.h"
+#include "config.h"
 
+#include "ochusha_private.h"
+#include "ochusha.h"
+
+#include "ochusha_ui.h"
 #include "bbs_thread_ui.h"
 #include "threadlist_filter.h"
 
@@ -1525,7 +1528,7 @@ void
 setup_default_threadlist_filter(OchushaApplication *application)
 {
   if (threadlist_filter_rule_default != NULL)
-    free(threadlist_filter_rule_default);
+    g_free(threadlist_filter_rule_default);
 
   if (application->user_default_filter.rule == NULL)
     {
@@ -1624,7 +1627,7 @@ threadlist_filter_check_thread(ThreadlistFilter *filter,
 	  break;
 
 	case THREADLIST_FILTER_RULE_SHOW_THREAD_SEEN_OFFLINE:
-	  if (application->conf.offline
+	  if (application->config.offline
 	      && thread->number_of_responses_read > 0)
 	    {
 #if DEBUG_GUI_MOST
@@ -1635,7 +1638,7 @@ threadlist_filter_check_thread(ThreadlistFilter *filter,
 	  break;
 
 	case THREADLIST_FILTER_RULE_HIDE_THREAD_NEVER_SEEN_OFFLINE:
-	  if (application->conf.offline
+	  if (application->config.offline
 	      && thread->number_of_responses_read == 0)
 	    {
 #if DEBUG_GUI_MOST
