@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_view.c,v 1.13 2003/06/07 14:35:31 fuyu Exp $
+ * $Id: threadlist_view.c,v 1.16 2003/06/21 16:23:53 fuyu Exp $
  */
 
 #include "config.h"
@@ -188,7 +188,7 @@ struct _ThreadlistInfo
 static GtkScrolledWindowClass *parent_class = NULL;
 static gint threadlist_view_signals[LAST_SIGNAL] =
   { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
-static gint tree_view_signals[TREE_VIEW_LAST_SIGNAL] = { 0, 0, 0 };
+static gint tree_view_signals[TREE_VIEW_LAST_SIGNAL] = { 0, 0, 0, 0 };
 static GtkItemFactory *popup_menu_item_factory = NULL;
 static GQuark list_entry_info_id;
 
@@ -370,18 +370,18 @@ threadlist_view_class_init(ThreadlistViewClass *klass)
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(ThreadlistViewClass, advance_view),
 		 NULL, NULL,
-		 ochusha_marshal_BOOLEAN__POINTER,
+		 ochusha_marshal_BOOLEAN__OBJECT,
 		 G_TYPE_BOOLEAN, 1,
-		 G_TYPE_POINTER);
+		 OCHUSHA_TYPE_BBS_THREAD);
   threadlist_view_signals[BACK_VIEW_SIGNAL] =
     g_signal_new("back_view",
 		 G_TYPE_FROM_CLASS(klass),
 		 G_SIGNAL_RUN_LAST,
 		 G_STRUCT_OFFSET(ThreadlistViewClass, back_view),
 		 NULL, NULL,
-		 ochusha_marshal_BOOLEAN__POINTER,
+		 ochusha_marshal_BOOLEAN__OBJECT,
 		 G_TYPE_BOOLEAN, 1,
-		 G_TYPE_POINTER);
+		 OCHUSHA_TYPE_BBS_THREAD);
 
   threadlist_view_signals[MOVE_CURSOR_SIGNAL] =
     g_signal_new("wrapped_move_cursor",
@@ -558,6 +558,7 @@ typedef enum
   THREADLIST_TITLE,
   THREADLIST_N_RESPONSES,
   THREADLIST_N_RESPONSES_READ,
+  THREADLIST_N_RESPONSES_UNREAD,
   THREADLIST_RANK_DIFFERENCE,
   THREADLIST_DATA,
   THREADLIST_N_COLUMNS
@@ -568,8 +569,10 @@ static void
 threadlist_view_init(ThreadlistView *view)
 {
   GtkWidget *tree_view;
+#if 0
   GtkTreeViewColumn *column;
   GtkCellRenderer *renderer;
+#endif
 
   GTK_WIDGET_SET_FLAGS(view, GTK_CAN_FOCUS | GTK_RECEIVES_DEFAULT);
 
@@ -603,71 +606,6 @@ threadlist_view_init(ThreadlistView *view)
   g_signal_connect(G_OBJECT(view->view), "button_release_event",
 		   G_CALLBACK(tree_view_button_release_event_cb), view);
 
-  renderer = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(_("Rank"), renderer,
-						    "text", THREADLIST_RANK,
-						    NULL);
-  gtk_tree_view_column_set_cell_data_func(column, renderer,
-					  decorate_threadlist_entry,
-					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
-  view->rank_column = column;
-
-  renderer = gtk_cell_renderer_toggle_new();
-  gtk_cell_renderer_toggle_set_radio(GTK_CELL_RENDERER_TOGGLE(renderer),
-				     FALSE);
-  column = gtk_tree_view_column_new_with_attributes(_("Mark"), renderer,
-					"active", THREADLIST_VISUAL_PARAMETER,
-					NULL);
-  gtk_tree_view_column_set_cell_data_func(column, renderer,
-					  active_set_cell_data_flags,
-					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
-  view->mark_column = column;
-
-  renderer = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(_("Thread Title"),
-						    renderer, "text",
-						    THREADLIST_TITLE, NULL);
-  gtk_tree_view_column_set_cell_data_func(column, renderer,
-					  decorate_threadlist_entry,
-					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
-  view->title_column = column;
-
-  renderer = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(_("#Res"),
-						    renderer, "text",
-						    THREADLIST_N_RESPONSES,
-						    NULL);
-  gtk_tree_view_column_set_cell_data_func(column, renderer,
-					  decorate_threadlist_entry,
-					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
-  view->n_res_column = column;
-
-  renderer = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(_("#Read"),
-						    renderer, "text",
-						    THREADLIST_N_RESPONSES_READ,
-						    NULL);
-  gtk_tree_view_column_set_cell_data_func(column, renderer,
-					  decorate_threadlist_entry,
-					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
-  view->n_read_column = column;
-
-  renderer = gtk_cell_renderer_text_new();
-  column = gtk_tree_view_column_new_with_attributes(_("Rank Difference"),
-						    renderer, "text",
-						    THREADLIST_RANK_DIFFERENCE,
-						    NULL);
-  gtk_tree_view_column_set_cell_data_func(column, renderer,
-					  text_set_cell_data_rank_diff,
-					  view, NULL);
-  gtk_tree_view_append_column(view->view, column);
-  view->rank_diff_column = column;
-
   view->model = NULL;
   view->model_is_opened = FALSE;
 
@@ -771,7 +709,8 @@ threadlist_view_title_not_equals(GtkTreeModel *model, gint column,
   if (key != NULL)
     {
       native_string = convert_string(utf8_to_native, key, -1);
-      fprintf(stderr, "threadlist_view_title_not_equals(): column=%d, key=\"%s\":",
+      fprintf(stderr,
+	      "threadlist_view_title_not_equals(): column=%d, key=\"%s\":",
 	      column, key);
       G_FREE(native_string);
     }
@@ -1437,6 +1376,9 @@ set_thread_values(ThreadlistView *view, GtkListStore *store,
 		  int last_rank, int new_rank, OchushaBBSThread *thread)
 {
   ThreadlistInfo *info = ensure_threadlist_info(thread);
+  int num_unread
+    = thread->number_of_responses_on_server - thread->number_of_responses_read;
+  num_unread = (num_unread >= 0 ? num_unread : 0);
 
   if (new_rank != RANK_UNSPECIFIED)
     {
@@ -1460,6 +1402,7 @@ set_thread_values(ThreadlistView *view, GtkListStore *store,
 		THREADLIST_TITLE, thread->title,
 		THREADLIST_N_RESPONSES, thread->number_of_responses_on_server,
 		THREADLIST_N_RESPONSES_READ, thread->number_of_responses_read,
+		THREADLIST_N_RESPONSES_UNREAD, num_unread,
 		THREADLIST_RANK_DIFFERENCE, rank_diff,
 		THREADLIST_DATA, thread,
 		-1);
@@ -1470,6 +1413,7 @@ set_thread_values(ThreadlistView *view, GtkListStore *store,
 		THREADLIST_TITLE, thread->title,
 		THREADLIST_N_RESPONSES, thread->number_of_responses_on_server,
 		THREADLIST_N_RESPONSES_READ, thread->number_of_responses_read,
+		THREADLIST_N_RESPONSES_UNREAD, num_unread,
 		THREADLIST_DATA, thread,
 		-1);
     }
@@ -1683,6 +1627,181 @@ text_set_cell_data_rank_diff(GtkTreeViewColumn *tree_column,
 }
 
 
+void
+threadlist_view_append_rank(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->rank_column == NULL);
+
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(_("Rank"), renderer,
+						    "text", THREADLIST_RANK,
+						    NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  decorate_threadlist_entry,
+					  view, NULL);
+  gtk_tree_view_append_column(view->view, column);
+  view->rank_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
+void
+threadlist_view_append_mark(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->mark_column == NULL);
+
+  renderer = gtk_cell_renderer_toggle_new();
+  gtk_cell_renderer_toggle_set_radio(GTK_CELL_RENDERER_TOGGLE(renderer),
+				     FALSE);
+  column = gtk_tree_view_column_new_with_attributes(_("Mark"), renderer,
+					"active", THREADLIST_VISUAL_PARAMETER,
+					NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  active_set_cell_data_flags,
+					  view, NULL);
+  gtk_tree_view_append_column(view->view, column);
+  view->mark_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
+void
+threadlist_view_append_title(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->title_column == NULL);
+
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(_("Thread Title"),
+						    renderer, "text",
+						    THREADLIST_TITLE, NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  decorate_threadlist_entry,
+					  view, NULL);
+  gtk_tree_view_append_column(view->view, column);
+  view->title_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
+void
+threadlist_view_append_number_of_responses(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->n_res_column == NULL);
+
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(_("#Res"),
+						    renderer, "text",
+						    THREADLIST_N_RESPONSES,
+						    NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  decorate_threadlist_entry,
+					  view, NULL);
+  gtk_tree_view_append_column(view->view, column);
+  view->n_res_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
+void
+threadlist_view_append_number_of_responses_read(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->n_read_column == NULL);
+
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(_("#Read"),
+						renderer, "text",
+						THREADLIST_N_RESPONSES_READ,
+						NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  decorate_threadlist_entry,
+					  view, NULL);
+  gtk_tree_view_append_column(view->view, column);
+  view->n_read_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
+void
+threadlist_view_append_number_of_responses_unread(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->n_unread_column == NULL);
+
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(_("#Unread"),
+						renderer, "text",
+						THREADLIST_N_RESPONSES_UNREAD,
+						NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  decorate_threadlist_entry,
+					  view, NULL);
+  gtk_tree_view_append_column(view->view, column);
+  view->n_unread_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
+void
+threadlist_view_append_rank_difference(ThreadlistView *view)
+{
+  GtkTreeViewColumn *column;
+  GtkCellRenderer *renderer;
+
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && view->rank_diff_column == NULL);
+
+  renderer = gtk_cell_renderer_text_new();
+  column = gtk_tree_view_column_new_with_attributes(_("Rank Difference"),
+						    renderer, "text",
+						    THREADLIST_RANK_DIFFERENCE,
+						    NULL);
+  gtk_tree_view_column_set_cell_data_func(column, renderer,
+					  text_set_cell_data_rank_diff,
+					  view, NULL);
+  gtk_tree_view_append_column(view->view, column);
+  view->rank_diff_column = column;
+
+  view->view_is_initialized = TRUE;
+}
+
+
+void
+threadlist_view_initialize_default_view(ThreadlistView *view)
+{
+  g_return_if_fail(IS_THREADLIST_VIEW(view) && !view->view_is_initialized);
+
+  threadlist_view_append_rank(view);
+  threadlist_view_append_mark(view);
+  threadlist_view_append_title(view);
+  threadlist_view_append_number_of_responses(view);
+  threadlist_view_append_number_of_responses_read(view);
+  threadlist_view_append_number_of_responses_unread(view);
+  threadlist_view_append_rank_difference(view);
+}
+
+
 gboolean
 threadlist_view_open(ThreadlistView *view)
 {
@@ -1691,6 +1810,9 @@ threadlist_view_open(ThreadlistView *view)
   g_return_val_if_fail(IS_THREADLIST_VIEW(view)
 		       && !view->model_is_opened, FALSE);
 
+  if (!view->view_is_initialized)
+    threadlist_view_initialize_default_view(view);
+
   if (view->model != NULL)
     g_object_unref(G_OBJECT(view->model));
 
@@ -1700,6 +1822,7 @@ threadlist_view_open(ThreadlistView *view)
 			     G_TYPE_STRING,	/* thread title */
 			     G_TYPE_INT,	/* number of responses read */
 			     G_TYPE_INT,	/* number of responses read */
+			     G_TYPE_INT,	/* number of responses unread*/
 			     G_TYPE_INT,	/* rank difference */
 			     G_TYPE_POINTER);	/* data (OchushaBBSThread *) */
 
