@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: threadlist_view.h,v 1.6 2003/05/17 08:14:47 fuyu Exp $
+ * $Id: threadlist_view.h,v 1.7 2003/06/21 16:23:53 fuyu Exp $
  */
 
 #ifndef _THREADLIST_VIEW_H_
@@ -79,6 +79,7 @@ struct _ThreadlistView
   GtkTreeViewColumn *title_column;
   GtkTreeViewColumn *n_res_column;
   GtkTreeViewColumn *n_read_column;
+  GtkTreeViewColumn *n_unread_column;
   GtkTreeViewColumn *rank_diff_column;
 
   GtkTreeSelection *selection;
@@ -93,6 +94,7 @@ struct _ThreadlistView
 
   OchushaBBSThread *recently_pointed_thread;
 
+  guint view_is_initialized	: 1;
   guint model_is_opened		: 1;
   guint use_native_search	: 1;
   guint being_searched		: 1;
@@ -191,6 +193,15 @@ typedef enum
 GType threadlist_view_get_type(void);
 GtkWidget *threadlist_view_new(void);
 
+void threadlist_view_append_rank(ThreadlistView *view);
+void threadlist_view_append_mark(ThreadlistView *view);
+void threadlist_view_append_title(ThreadlistView *view);
+void threadlist_view_append_number_of_responses(ThreadlistView *view);
+void threadlist_view_append_number_of_responses_read(ThreadlistView *view);
+void threadlist_view_append_number_of_responses_unread(ThreadlistView *view);
+void threadlist_view_append_rank_difference(ThreadlistView *view);
+
+void threadlist_view_initialize_default_view(ThreadlistView *view);
 
 gboolean threadlist_view_open(ThreadlistView *view);
 void threadlist_view_close(ThreadlistView *view,
