@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: htmlutils.c,v 1.2 2003/06/06 19:38:04 fuyu Exp $
+ * $Id: htmlutils.c,v 1.4 2003/07/15 05:37:42 fuyu Exp $
  */
 
 #include "config.h"
@@ -43,15 +43,6 @@
 #include <string.h>
 
 
-#define ENABLE_FAKE_ANCHOR	1
-#define ENABLE_SKIP_IME_NU	1
-
-#define DEBUG_PARSER		0
-#define DEBUG_PARSER_MOST	0
-#define DEBUG_ICONV		0
-#define DEBUG_ICONV_MOST	0
-
-
 static const char *mempbrk(const char *string, const char *charset,
 			   size_t len);
 static const char *process_tag(ElementHandler *handler, void *context,
@@ -531,6 +522,67 @@ parse_text(ElementHandler *handler, void *context, const char *text, int len)
 	       */
 	      while (1)
 		{
+#if ENABLE_MS_IZONMOJI
+		  do
+		    {
+		      size = iconv(handler->converter, &inbuf, &inbytesleft,
+				   &outbuf, &outbytesleft);
+		      if (size == -1 && errno == EILSEQ
+			  && handler->helper != NULL)
+			size = (*handler->helper)(&inbuf, &inbytesleft,
+						  &outbuf, &outbytesleft);
+		      else
+			break;
+		    } while (size != -1);
+
+		  if (size != -1)
+		    break;	/* ｪλ */
+
+		  if (errno == E2BIG)
+		    {
+		      size_t old_size = buffer_size;
+		      /* allocate 2 times larger buffer in the heap. */
+		      buffer_size *= 2;
+		      if (buffer != default_buffer)
+			buffer = (gchar *)G_REALLOC(buffer, buffer_size);
+		      else
+			{
+			  gchar *old_buffer = buffer;
+			  buffer = (gchar *)G_MALLOC(buffer_size);
+			  if (buffer != NULL)
+			    memcpy(buffer, old_buffer,
+				   old_size - outbytesleft);
+			}
+		      outbytesleft += old_size;
+		      outbuf = buffer + buffer_size - outbytesleft;
+		    }
+		  else
+		    {
+#if DEBUG_ICONV
+		      /*
+		       * TODO: ϽĥäѴ
+		       *       ҤȤޤ̵롣ġĤȤconverter
+		       *       //IGNOREꤷƤ餳ˤ
+		       *       ϤʤΤġġ
+		       */
+		      int i;
+		      int count = (inbytesleft > 8) ? 8 : inbytesleft;
+		      fprintf(stderr, "iconv failed: %s(%d).\n",
+			      strerror(errno), errno);
+		      fprintf(stderr, "failed sequence:");
+		      for (i = 0; i < count; i++)
+			fprintf(stderr, " 0x%02x\n", inbuf[i] & 0xff);
+		      fprintf(stderr, "\n");
+#endif
+		      if (buffer != default_buffer)
+			G_FREE(buffer);
+#if DEBUG_ICONV_MOST
+		      abort();
+#else
+		      return NULL;
+#endif
+		    }
+#else
 		  size = iconv(handler->converter,
 			       &inbuf, &inbytesleft, &outbuf, &outbytesleft);
 		  if (size == -1)
@@ -573,7 +625,7 @@ parse_text(ElementHandler *handler, void *context, const char *text, int len)
 #endif
 			  if (buffer != default_buffer)
 			    G_FREE(buffer);
-#if DEBUG_ICONV
+#if DEBUG_ICONV_MOST
 			  abort();
 #else
 			  return NULL;
@@ -582,6 +634,7 @@ parse_text(ElementHandler *handler, void *context, const char *text, int len)
 		    }
 		  else
 		    break;
+#endif	/* ENABLE_MS_IZONMOJI */
 		}
 	      (*handler->characters)(context, buffer,
 				     buffer_size - outbytesleft);
@@ -857,7 +910,8 @@ process_entity_ref(ElementHandler *handler, void *context, const char *head,
       {
 	gchar *utf8_entity = NULL;
 	gchar broken_ref_buffer[DEFAULT_BUFFER_SIZE];
-	utf8_entity = convert_string(handler->converter, entity, -1);
+	utf8_entity = convert_string(handler->converter, handler->helper,
+				     entity, -1);
 	if (utf8_entity != NULL)
 	  snprintf(broken_ref_buffer, DEFAULT_BUFFER_SIZE, "&%s%s",
 		   utf8_entity, (*ref_tail == ';' ? ";" : ""));
@@ -1079,7 +1133,8 @@ append_characters(void *ctx, const gchar *ch, int len)
 
 
 gchar *
-simple_string_canon(const char *src, int len, iconv_t converter)
+simple_string_canon(const char *src, int len,
+		    iconv_t converter, iconv_helper *helper)
 {
   gchar buffer[DEFAULT_BUFFER_SIZE];
   const char *cur_pos;
@@ -1088,6 +1143,7 @@ simple_string_canon(const char *src, int len, iconv_t converter)
   ElementHandler handler =
     {
       converter,
+      helper,
       NULL,
       NULL,
       NULL,
