@@ -23,13 +23,18 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_async_buffer.c,v 1.1.1.1 2003/05/10 16:34:28 fuyu Exp $
+ * $Id: ochusha_async_buffer.c,v 1.5 2003/06/07 14:35:31 fuyu Exp $
  */
 
+#include "config.h"
+
+#include "ochusha_private.h"
+
 #include "ochusha_async_buffer.h"
 
 #include <glib.h>
 
+#include <errno.h>
 #include <pthread.h>	/* GThread֤Ĥ */
 
 #include <stdio.h>
@@ -40,6 +45,40 @@
 #define OCHUSHA_INITIAL_ASYNC_BUFFER_SIZE	4096
 
 
+static void ochusha_async_buffer_class_init(OchushaAsyncBufferClass *klass);
+static void ochusha_async_buffer_init(OchushaAsyncBuffer *buffer);
+static void ochusha_async_buffer_finalize(GObject *object);
+
+
+GType
+ochusha_async_buffer_get_type(void)
+{
+  static GType async_buffer_type = 0;
+
+  if (async_buffer_type == 0)
+    {
+      static const GTypeInfo async_buffer_info =
+	{
+	  sizeof(OchushaAsyncBufferClass),
+	  NULL, /* base_init */
+	  NULL, /* base_finalize */
+	  (GClassInitFunc)ochusha_async_buffer_class_init,
+	  NULL, /* class_finalize */
+	  NULL, /* class_data */
+	  sizeof(OchushaAsyncBuffer),
+	  0,	/* n_preallocs */
+	  (GInstanceInitFunc)ochusha_async_buffer_init,
+	};
+
+      async_buffer_type = g_type_register_static(G_TYPE_OBJECT,
+						 "OchushaAsyncBuffer",
+						 &async_buffer_info, 0);
+    }
+
+  return async_buffer_type;
+}
+
+
 struct _BufferSyncObject
 {
   pthread_mutex_t mutex;
@@ -47,11 +86,13 @@ struct _BufferSyncObject
 };
 
 
-static GObjectClass *parent_class = NULL;
+static BufferSyncObject *get_sync_object(void);
 
+static GObjectClass *parent_class = NULL;
 static pthread_mutex_t ochusha_async_buffer_global_lock;
 static pthread_cond_t ochusha_async_buffer_global_condition;
 
+
 /*
  * pthreadϢΥ֥ȤλȤΤƤƧߤʤΤǡѤ
  * ݤ롣
@@ -94,12 +135,40 @@ static GSList *ochusha_async_buffer_list = NULL;
       abort();								\
     }
 
+#define LOCK_BUFFER_LOCK(buffer)					\
+  do {									\
+    int result = pthread_mutex_trylock(&buffer->sync_object->mutex);	\
+    if (result != 0)							\
+      {									\
+	if (result == EBUSY)						\
+	  {								\
+            LOCK_GLOBAL_LOCK;						\
+	    buffer->number_of_lock_waiters++;				\
+            UNLOCK_GLOBAL_LOCK;						\
+	    if (pthread_mutex_lock(&buffer->sync_object->mutex) != 0)	\
+	      {								\
+		fprintf(stderr, "Couldn't lock a mutex.\n");		\
+		abort();						\
+	      }								\
+            LOCK_GLOBAL_LOCK;						\
+	    buffer->number_of_lock_waiters--;				\
+            UNLOCK_GLOBAL_LOCK;						\
+	  }								\
+	else								\
+	  {								\
+	    fprintf(stderr, "Couldn't trylock a mutex.\n");		\
+	    abort();							\
+	  }								\
+      }									\
+  } while (0)
+#if 0
 #define LOCK_BUFFER_LOCK(buffer)				\
   if (pthread_mutex_lock(&buffer->sync_object->mutex) != 0)	\
     {								\
       fprintf(stderr, "Couldn't lock a mutex.\n");		\
       abort();							\
     }
+#endif
 
 #define UNLOCK_BUFFER_LOCK(buffer)				\
   if (pthread_mutex_unlock(&buffer->sync_object->mutex) != 0)	\
@@ -131,42 +200,6 @@ static GSList *ochusha_async_buffer_list = NULL;
     }
 
 
-static void ochusha_async_buffer_class_init(OchushaAsyncBufferClass *klass);
-static void ochusha_async_buffer_init(OchushaAsyncBuffer *buffer);
-static void ochusha_async_buffer_finalize(GObject *object);
-
-static BufferSyncObject *get_sync_object(void);
-
-
-GType
-ochusha_async_buffer_get_type(void)
-{
-  static GType async_buffer_type = 0;
-
-  if (async_buffer_type == 0)
-    {
-      static const GTypeInfo async_buffer_info =
-	{
-	  sizeof(OchushaAsyncBufferClass),
-	  NULL, /* base_init */
-	  NULL, /* base_finalize */
-	  (GClassInitFunc)ochusha_async_buffer_class_init,
-	  NULL, /* class_finalize */
-	  NULL, /* class_data */
-	  sizeof(OchushaAsyncBuffer),
-	  0,	/* n_preallocs */
-	  (GInstanceInitFunc)ochusha_async_buffer_init,
-	};
-
-      async_buffer_type = g_type_register_static(G_TYPE_OBJECT,
-						 "OchushaAsyncBuffer",
-						 &async_buffer_info, 0);
-    }
-
-  return async_buffer_type;
-}
-
-
 static void
 ochusha_async_buffer_class_init(OchushaAsyncBufferClass *klass)
 {
@@ -193,6 +226,7 @@ static void
 ochusha_async_buffer_init(OchushaAsyncBuffer *buffer)
 {
   buffer->fixed = FALSE;
+  buffer->number_of_lock_waiters = 0;
 }
 
 
@@ -200,16 +234,19 @@ static void
 ochusha_async_buffer_finalize(GObject *object)
 {
   OchushaAsyncBuffer *buffer = OCHUSHA_ASYNC_BUFFER(object);
-#if DEBUG_ASYNC_BUFFER
-  fprintf(stderr, "ochusha_async_buffer_finalize: OchushaAsyncBuffer(0x%x)\n",
-	  (int)object);
+#if DEBUG_OCHUSHA_MEMORY_USAGE
+  fprintf(stderr, "ochusha_async_buffer_finalize: OchushaAsyncBuffer(%p)\n",
+	  object);
 #endif
   if (buffer->destructor != NULL)
-    (*buffer->destructor)(buffer, buffer->user_data);
+    (*buffer->destructor)(buffer);
 
   LOCK_GLOBAL_LOCK;
   if (buffer->sync_object != NULL)
-    sync_object_list = g_slist_append(sync_object_list, buffer->sync_object);
+    {
+      sync_object_list = g_slist_append(sync_object_list, buffer->sync_object);
+      buffer->sync_object = NULL;
+    }
   ochusha_async_buffer_list = g_slist_remove(ochusha_async_buffer_list, buffer);
   UNLOCK_GLOBAL_LOCK;
 
@@ -234,7 +271,7 @@ get_sync_object(void)
       return sync_object;
     }
 
-  sync_object = (BufferSyncObject *)g_malloc(sizeof(BufferSyncObject));
+  sync_object = (BufferSyncObject *)G_MALLOC(sizeof(BufferSyncObject));
   if (pthread_mutex_init(&sync_object->mutex, NULL) != 0)
     {
       fprintf(stderr, "Couldn't init a mutex.\n");
@@ -250,8 +287,7 @@ get_sync_object(void)
 
 
 OchushaAsyncBuffer *
-ochusha_async_buffer_new(char *buffer, int length,
-			 gpointer user_data, DestructFunc *destructor)
+ochusha_async_buffer_new(char *buffer, int length, DestructFunc *destructor)
 {
   BufferSyncObject *sync_object;
   OchushaAsyncBuffer *buf
@@ -268,7 +304,7 @@ ochusha_async_buffer_new(char *buffer, int length,
   if (buffer == NULL && destructor == NULL)
     {
       if (length != 0)
-	buffer = (char *)g_malloc(length);
+	buffer = (char *)G_MALLOC(length);
       destructor = ochusha_async_buffer_free_when_finished;
       buf->length = 0;
     }
@@ -281,9 +317,12 @@ ochusha_async_buffer_new(char *buffer, int length,
   buf->buffer_length = length;	/* XXX: Ǥ餦٤ġġ*/
 
   buf->sync_object = sync_object;
-  buf->user_data = user_data;
   buf->destructor = destructor;
 
+#if DEBUG_MEMORY_USAGE
+  fprintf(stderr, "ochusha_async_buffer_new(): OchushaAsyncBuffer(%p)\n", buf);
+#endif
+
   return buf;
 }
 
@@ -314,9 +353,9 @@ ochusha_async_buffer_active_ref(OchushaAsyncBuffer *buffer, const char *user)
 {
   gboolean result;
 
-#if DEBUG_ASYNC_BUFFER_MOST
-  fprintf(stderr, "OchushaAsyncBuffer(0x%x) is actively refered by %s\n",
-	  (int)buffer, user);
+#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
+  fprintf(stderr, "OchushaAsyncBuffer(%p) is actively refered by %s\n",
+	  buffer, user);
 #endif
   g_object_ref(G_OBJECT(buffer));
 
@@ -350,10 +389,10 @@ ochusha_async_buffer_active_ref(OchushaAsyncBuffer *buffer, const char *user)
 void
 ochusha_async_buffer_active_unref(OchushaAsyncBuffer *buffer, const char *user)
 {
-#if DEBUG_ASYNC_BUFFER_MOST
+#if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr,
-	  "OchushaAsyncBuffer(0x%x) actively refered by %s is unrefered\n",
-	  (int)buffer, user);
+	  "OchushaAsyncBuffer(%p) actively refered by %s is unrefered\n",
+	  buffer, user);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -393,8 +432,8 @@ void
 ochusha_async_buffer_suspend(OchushaAsyncBuffer *buffer)
 {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "suspend threads refering OchushaAsyncBuffer(0x%x)\n",
-	  (int)buffer);
+  fprintf(stderr, "suspend threads refering OchushaAsyncBuffer(%p)\n",
+	  buffer);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -452,7 +491,7 @@ ochusha_async_buffer_terminate(OchushaAsyncBuffer *buffer)
 {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr,
-	  "ochusha_async_buffer_terminate: buffer=0x%x\n", (int)buffer);
+	  "ochusha_async_buffer_terminate: buffer=%p\n", buffer);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -539,9 +578,6 @@ gboolean
 ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, int length,
 			    const char *user)
 {
-  char *new_buf;
-  gboolean result;
-
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr, "\"%s\" called ochusha_async_buffer_resize\n", user);
 #endif
@@ -554,36 +590,32 @@ ochusha_async_buffer_resize(OchushaAsyncBuffer *buffer, int length,
       abort();
     }
 
-  new_buf = (char *)realloc((void *)buffer->buffer, length);
-  if (new_buf == NULL)
-    result = FALSE;
-  else
-    {
-      buffer->buffer = new_buf;
-      buffer->buffer_length = length;
-      result = TRUE;
-    }
+  buffer->buffer = (char *)G_REALLOC((void *)buffer->buffer, length);
+  buffer->buffer_length = length;
 
   UNLOCK_BUFFER_LOCK(buffer);
 
-  return ochusha_async_buffer_broadcast(buffer, user) && result;
+  return ochusha_async_buffer_broadcast(buffer, user);
 }
 
 
 /*
- * Out of memory⤷ϥХåեTERMINATED֤ξFALSE֤
+ * ochusha_async_buffer_ensure_free_space:
+ *
+ * ХåեlengthʾĹĤȤݾڤ롣
+ *
+ * ХåեTERMINATED֤ξFALSE֤
  */
 gboolean
 ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer, int length,
 				       const char *user)
 {
-  char *new_buf;
   gboolean result = TRUE;
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr,
-	  "ochusha_async_buffer_ensure_free_space(0x%x, %d, \"%s\") called.\n",
-	  (int)buffer, length, user);
+	  "ochusha_async_buffer_ensure_free_space(%p, %d, \"%s\") called.\n",
+	  buffer, length, user);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -621,19 +653,10 @@ ochusha_async_buffer_ensure_free_space(OchushaAsyncBuffer *buffer, int length,
       size_t new_buf_len
 	= (buffer->buffer_length > 0)
 	? (buffer->buffer_length * 2) : OCHUSHA_INITIAL_ASYNC_BUFFER_SIZE;
-      new_buf = (char *)realloc((void *)buffer->buffer, new_buf_len);
-      if (new_buf != NULL)
-	{
-	  buffer->buffer = new_buf;
-	  buffer->buffer_length = new_buf_len;
-	}
-      else
-	{
-	  result = FALSE;
-	  break;
-	}
+      buffer->buffer = (char *)G_REALLOC((void *)buffer->buffer, new_buf_len);
+      buffer->buffer_length = new_buf_len;
     }
-  result = (buffer->state != OCHUSHA_ASYNC_BUFFER_TERMINATED) && result;
+  result = (buffer->state != OCHUSHA_ASYNC_BUFFER_TERMINATED);
   UNLOCK_BUFFER_LOCK(buffer);
 
   return result;
@@ -683,8 +706,8 @@ ochusha_async_buffer_check_active(OchushaAsyncBuffer *buffer, const char *user)
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
   fprintf(stderr,
-	  "ochusha_async_buffer_check_active(0x%x, \"%s\")\n",
-	  (int)buffer, user);
+	  "ochusha_async_buffer_check_active(%p, \"%s\")\n",
+	  buffer, user);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -729,8 +752,8 @@ ochusha_async_buffer_signal(OchushaAsyncBuffer *buffer, const char *user)
   gboolean result;
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_signal(0x%x, \"%s\")\n",
-	  (int)buffer, user);
+  fprintf(stderr, "ochusha_async_buffer_signal(%p, \"%s\")\n",
+	  buffer, user);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -778,8 +801,8 @@ ochusha_async_buffer_broadcast(OchushaAsyncBuffer *buffer, const char *user)
   gboolean result;
 
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_broadcast(0x%x, \"%s\")\n",
-	  (int)buffer, user);
+  fprintf(stderr, "ochusha_async_buffer_broadcast(%p, \"%s\")\n",
+	  buffer, user);
 #endif
 
   LOCK_BUFFER_LOCK(buffer);
@@ -824,8 +847,8 @@ gboolean
 ochusha_async_buffer_wait(OchushaAsyncBuffer *buffer, const char *user)
 {
 #if DEBUG_OCHUSHA_ASYNC_BUFFER_MOST
-  fprintf(stderr, "ochusha_async_buffer_wait(0x%x, \"%s\")\n",
-	  (int)buffer, user);
+  fprintf(stderr, "ochusha_async_buffer_wait(%p, \"%s\")\n",
+	  buffer, user);
 #endif
 
   while (buffer->state == OCHUSHA_ASYNC_BUFFER_SUSPENDED)
@@ -861,10 +884,24 @@ ochusha_async_buffer_wait(OchushaAsyncBuffer *buffer, const char *user)
 }
 
 
+/*
+ * ochusha_async_buffer_is_busy:
+ *
+ * bufferΥåcontentionäƤȤˤTRUE֤
+ */
+gboolean
+ochusha_async_buffer_is_busy(OchushaAsyncBuffer *buffer,
+			     const char *user)
+{
+  g_return_val_if_fail(OCHUSHA_IS_ASYNC_BUFFER(buffer), FALSE);
+
+  return buffer->number_of_lock_waiters > 0;
+}
+
+
 void
-ochusha_async_buffer_free_when_finished(OchushaAsyncBuffer *buffer,
-					gpointer unused)
+ochusha_async_buffer_free_when_finished(OchushaAsyncBuffer *buffer)
 {
   if (buffer->buffer != NULL)
-    free((void *)buffer->buffer);
+    G_FREE((void *)buffer->buffer);
 }
