@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_table.c,v 1.4 2003/06/07 14:35:31 fuyu Exp $
+ * $Id: ochusha_bbs_table.c,v 1.6 2003/06/20 18:10:06 fuyu Exp $
  */
 
 #include "config.h"
@@ -905,7 +905,7 @@ ochusha_bbs_table_read_boardlist_xml(OchushaBBSTable *table,
 
   xmlInitParser();
 
-  sax_handler.initialized = 0;
+  memset(&sax_handler, 0, sizeof(xmlSAXHandler));
   initxmlDefaultSAXHandler(&sax_handler, TRUE);
 
   sax_handler.getEntity = getEntityHandler;
@@ -946,6 +946,9 @@ write_board(gpointer data, gpointer user_data)
   WriteBoardlistArgs *args = (WriteBoardlistArgs *)user_data;
   FILE *boardlist_xml = args->boardlist_xml;
 
+  if (board->killed)
+    return;
+
   fprintf(boardlist_xml, "      <board>\n");
 
   g_signal_emit_by_name(G_OBJECT(board),
@@ -972,6 +975,9 @@ write_category(gpointer data, gpointer user_data)
   WriteBoardlistArgs *args = (WriteBoardlistArgs *)user_data;
   FILE *boardlist_xml = args->boardlist_xml;
 
+  if (category->killed)
+    return;
+
   fprintf(boardlist_xml, "    <category>\n");
 
   g_signal_emit_by_name(G_OBJECT(category),
