@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bbs_thread.c,v 1.8 2003/06/22 10:11:10 fuyu Exp $
+ * $Id: ochusha_bbs_thread.c,v 1.9 2003/07/13 14:53:51 fuyu Exp $
  */
 
 #include "config.h"
@@ -391,6 +391,15 @@ ochusha_bbs_thread_get_response_character_encoding(OchushaBBSThread *thread)
 }
 
 
+iconv_helper *
+ochusha_bbs_thread_get_response_iconv_helper(OchushaBBSThread *thread)
+{
+  g_return_val_if_fail(OCHUSHA_IS_BBS_THREAD(thread)
+		       && OCHUSHA_IS_BULLETIN_BOARD(thread->board), NULL);
+  return ochusha_bulletin_board_get_response_iconv_helper(OCHUSHA_BULLETIN_BOARD(thread->board));
+}
+
+
 gboolean
 ochusha_bbs_thread_parse_responses(OchushaBBSThread *thread,
 				   OchushaAsyncBuffer *buffer,
