@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_2ch.c,v 1.5 2003/06/07 14:35:32 fuyu Exp $
+ * $Id: ochusha_board_2ch.c,v 1.7 2003/07/13 14:53:51 fuyu Exp $
  */
 
 #include "config.h"
@@ -82,7 +82,8 @@ static gboolean ochusha_board_2ch_refresh_threadlist(
 
 static const char *ochusha_board_2ch_get_response_character_encoding(
 						OchushaBulletinBoard *board);
-
+static iconv_helper *ochusha_board_2ch_get_response_iconv_helper(
+						OchushaBulletinBoard *board);
 
 GType
 ochusha_board_2ch_get_type(void)
@@ -134,6 +135,8 @@ ochusha_board_2ch_class_init(OchushaBoard2chClass *klass)
 
   b_class->get_response_character_encoding
     = ochusha_board_2ch_get_response_character_encoding;
+  b_class->get_response_iconv_helper
+    = ochusha_board_2ch_get_response_iconv_helper;
 }
 
 
@@ -310,6 +313,22 @@ ochusha_board_2ch_get_response_character_encoding(OchushaBulletinBoard *board)
 }
 
 
+static iconv_helper *
+ochusha_board_2ch_get_response_iconv_helper(OchushaBulletinBoard *board)
+{
+  g_return_val_if_fail(OCHUSHA_IS_BOARD_2CH(board), NULL);
+
+  switch (((OchushaBulletinBoard *)board)->bbs_type)
+    {
+    case OCHUSHA_BBS_TYPE_2CH:
+      return cp932_to_utf8_helper;
+
+    default:
+      return NULL;
+    }
+}
+
+
 OchushaBulletinBoard *
 ochusha_board_2ch_new(const gchar *name, const char *base_url)
 {
@@ -370,6 +389,17 @@ collect_dropped_thread(gpointer data, gpointer user_data)
 }
 
 
+static void
+undo_thread_ref(gpointer data, gpointer user_data)
+{
+  OchushaThread2ch *thread_2ch = OCHUSHA_THREAD_2CH(data);
+  if (thread_2ch->alive)
+    g_object_unref(G_OBJECT(data));	/* Ƥ륹thread_list
+					 * Ѥ;פrefƤ롣
+					 */
+}
+
+
 /*
  * ochusha_board_2ch_refresh_threadlist:
  *
@@ -480,7 +510,8 @@ ochusha_board_2ch_refresh_threadlist(OchushaBulletinBoard *board,
 	    if (thread == NULL)
 	      {
 		thread_title = simple_string_canon(title_pos, title_len,
-						   converter);
+						   converter,
+						   cp932_to_utf8_helper);
 		thread = ochusha_bulletin_board_bbs_thread_new(board,
 							       dat_file,
 							       thread_title);
@@ -547,10 +578,18 @@ ochusha_board_2ch_refresh_threadlist(OchushaBulletinBoard *board,
 
   ochusha_async_buffer_active_unref(buffer, "ochusha_board_2ch.c: ochusha_board_2ch_refresh_threadlist");
 
-  g_slist_foreach(old_thread_list, collect_dropped_thread, NULL);
-  g_slist_free(old_thread_list);
-
-  board->thread_list = thread_list;
+  if (result)
+    {
+      g_slist_foreach(old_thread_list, collect_dropped_thread, NULL);
+      g_slist_free(old_thread_list);
+      board->thread_list = thread_list;
+    }
+  else
+    {
+      /* 顼ϸŤΤͥ */
+      g_slist_foreach(old_thread_list, undo_thread_ref, NULL);
+      g_slist_free(thread_list);
+    }
 
   iconv_close(converter);
 
