@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_bulletin_board.c,v 1.8 2003/06/20 18:10:06 fuyu Exp $
+ * $Id: ochusha_bulletin_board.c,v 1.9 2003/07/13 14:53:51 fuyu Exp $
  */
 
 #include "config.h"
@@ -541,6 +541,20 @@ ochusha_bulletin_board_get_response_character_encoding(
 }
 
 
+iconv_helper *
+ochusha_bulletin_board_get_response_iconv_helper(OchushaBulletinBoard *board)
+{
+  OchushaBulletinBoardClass *klass;
+
+  g_return_val_if_fail(OCHUSHA_BULLETIN_BOARD(board), NULL);
+
+  klass = OCHUSHA_BULLETIN_BOARD_GET_CLASS(board);
+  g_return_val_if_fail(klass->get_response_iconv_helper != NULL, NULL);
+
+  return (*klass->get_response_iconv_helper)(board);
+}
+
+
 /*
  * boardlist.xmlϢ
  */
