@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.5 2003/06/10 21:02:55 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.6 2003/06/22 14:38:47 fuyu Exp $
  */
 
 #include "config.h"
@@ -465,6 +465,8 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   const char *url = args->url;
   const char *if_modified_since = args->if_modified_since;
   const char *error_message = NULL;
+  const char *connection;
+  gboolean connection_closed = FALSE;
 
   /* ʲϸľɬ */
 #if DEBUG_NETWORK
@@ -494,10 +496,11 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
    *       ΤޤǤϤޤǤϥʤ
    */
   ghttp_set_type(request, ghttp_type_head);
+  ghttp_set_header(request, http_hdr_Accept_Encoding, "deflate, gzip");
+
 #if DISABLE_PERSISTENT_CONNECTION
   ghttp_set_header(request, http_hdr_Connection, "close");
 #endif
-  ghttp_set_header(request, http_hdr_Accept_Encoding, "deflate, gzip");
 
   if (if_modified_since != NULL
       && ochusha_config_cache_file_exist(broker->config, url))
@@ -555,6 +558,17 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       nanosleep(&polling_interval, NULL);
     }
 
+  /* Persistent connection򥵥ݡȤʤФäݤ*/
+  connection = ghttp_get_header(request, http_hdr_Connection);
+  if (connection != NULL)
+    {
+      connection_closed = TRUE;
+#if DEBUG_NETWORK
+      if (strcmp(connection, "close") != 0)
+	fprintf(stderr, "Connection: %s\n", connection);
+#endif	
+    }
+
 #if DEBUG_NETWORK_MOST
   {
     char **hdrs;
@@ -587,7 +601,8 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
       else
 	fprintf(stderr, "Unknown error.\n");
 #endif
-      if (status_code != 304)
+      if (status_code != 304
+	  && buffer->state != OCHUSHA_ASYNC_BUFFER_TERMINATED)
 	{
 	  error_message = _("Unexpected response for HEAD request.");
 	  goto error_exit;
@@ -598,7 +613,6 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
        *       顼ǤϤʤcallerϥåǤΤȤ
        *       ȤȤǤ롣
        */
-
       ghttp_request_destroy(request);
       args->request = NULL;
       return FALSE;
@@ -614,7 +628,14 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
   request = ghttp_request_new();
   args->request = request;
 #else
-  ghttp_clean(request);
+  if (connection_closed)
+    {
+      ghttp_request_destroy(request);
+      request = ghttp_request_new();
+      args->request = request;
+    }
+  else
+    ghttp_clean(request);
 #endif
 
   ghttp_set_uri(request, (char *)url);
@@ -730,7 +751,7 @@ http_read_from_url(OchushaNetworkBroker *broker, OchushaAsyncBuffer *buffer)
 		}
 	    }
 	  else if (!ochusha_async_buffer_append_data(buffer, body, body_len,
-					"network.c: http_read_from_url"))
+			"ochusha_network_broker.c: http_read_from_url"))
 	    {	/* content_encoding == IDENTITY_ENCODING */
 	      g_signal_emit(G_OBJECT(broker),
 			    broker_signals[ACCESS_TERMINATED_SIGNAL],
@@ -953,7 +974,7 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 
   if (len > 0)
     {	/* å夢ξ */
-
+      const char *connection;
       /* MEMO:
        * ޤHEADꥯȤĹåĹӡ
        * åä麹ʬԡ
@@ -1108,7 +1129,20 @@ try_update_cache(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  request = ghttp_request_new();
 	  args->request = request;
 #else
-	  ghttp_clean(request);
+	  /* Persistent connection򥵥ݡȤʤФäݤ */
+	  connection = ghttp_get_header(request, http_hdr_Connection);
+	  if (connection != NULL)
+	    {
+#if DEBUG_NETWORK
+	      if (strcmp(connection, "close") != 0)
+		fprintf(stderr, "Connection: %s\n", connection);
+#endif	
+	      ghttp_request_destroy(request);
+	      request = ghttp_request_new();
+	      args->request = request;
+	    }
+	  else
+	    ghttp_clean(request);
 #endif
 
 	  ghttp_set_uri(request, url);
@@ -1579,13 +1613,20 @@ refresh_cache_after_read(WorkerThread *employee, OchushaAsyncBuffer *buffer)
 	  else
 	    {
 	      /* ̤˼ */
-	      g_signal_emit(G_OBJECT(args->broker),
-			    broker_signals[ACCESS_FAILED_SIGNAL],
-			    0,
-			    buffer,
-			    OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
-			    _("Couldn't read data via the network."),
-			    &signal_result);
+	      if (buffer->state != OCHUSHA_ASYNC_BUFFER_TERMINATED)
+		g_signal_emit(G_OBJECT(args->broker),
+			      broker_signals[ACCESS_FAILED_SIGNAL],
+			      0,
+			      buffer,
+			      OCHUSHA_NETWORK_BROKER_FAILURE_REASON_UNKNOWN,
+			      _("Couldn't read data via the network."),
+			      &signal_result);
+	      else
+		g_signal_emit(G_OBJECT(args->broker),
+			      broker_signals[ACCESS_TERMINATED_SIGNAL],
+			      0,
+			      buffer,
+			      &signal_result);
 	    }
 	}
     }
