@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_private.h,v 1.2 2003/06/06 19:38:04 fuyu Exp $
+ * $Id: ochusha_private.h,v 1.3 2003/07/15 05:37:42 fuyu Exp $
  */
 
 #ifndef _OCHUSHA_PRIVATE_H_
@@ -32,6 +32,19 @@
 #include "config.h"
 
 #define TRACE_MEMORY_USAGE	0
+
+#define ENABLE_FAKE_ANCHOR	1
+#define ENABLE_SKIP_IME_NU	1
+#define ENABLE_MS_IZONMOJI	1
+
+
+#define DEBUG_PARSER		0
+#define DEBUG_PARSER_MOST	0
+#define DEBUG_ICONV		0
+#define DEBUG_ICONV_MOST	0
+#define DEBUG_UTILS		0
+
+
 #include "utils.h"
 
 #include <glib.h>
