@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_thread_2ch.c,v 1.7 2003/06/12 04:13:38 fuyu Exp $
+ * $Id: ochusha_thread_2ch.c,v 1.9 2003/06/22 13:21:43 fuyu Exp $
  */
 
 #include "config.h"
@@ -76,6 +76,7 @@ static gboolean ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 						   int start, int number,
 						   StartThreadCallback *st_cb,
 						   EachResponseCallback *r_cb,
+						   BrokenResponseCallback *bcb,
 						   EndThreadCallback *end_cb,
 						   gpointer callback_data);
 
@@ -683,6 +684,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 				   int start, int number,
 				   StartThreadCallback *start_thread_cb,
 				   EachResponseCallback *each_response_cb,
+				   BrokenResponseCallback *broken_response_cb,
 				   EndThreadCallback *end_thread_cb,
 				   gpointer callback_data)
 {
@@ -799,9 +801,29 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 	      }
 	    responses[i] = offset;
 
+	    i++;
+#if 1
+	    if (i > start)
+	      {
+		result = TRUE;
+		if (title_pos != NULL)
+		  {
+		    if (each_response_cb != NULL)
+		      result = (*each_response_cb)(thread, i, &response,
+						   callback_data);
+		  }
+		else
+		  {
+		    if (broken_response_cb != NULL)
+		      result = (*broken_response_cb)(thread, i, callback_data);
+		  }
+		rest_of_responses--;
+		if (!result)
+		  break;
+	      }
+#else
 	    if (title_pos != NULL)
 	      {
-		i++;
 		if (i > start)
 		  {
 		    if (each_response_cb != NULL)
@@ -812,6 +834,7 @@ ochusha_thread_2ch_parse_responses(OchushaBBSThread *thread,
 		if (!result)
 		  break;
 	      }
+#endif
 
 	    offset = (eol_pos + 1) - buffer_top;
 	    buffer_top = (char *)buffer->buffer;
