@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: utils.h,v 1.3 2003/06/06 19:38:05 fuyu Exp $
+ * $Id: utils.h,v 1.4 2003/07/13 14:53:52 fuyu Exp $
  */
 
 #ifndef _UTILS_H_
@@ -43,12 +43,18 @@ extern iconv_t utf8_to_native;
 /*
  * utilityؿʤ
  */
+typedef size_t iconv_helper(const char **inbuf, size_t *inbytesleft,
+			    char **outbuf, size_t *outbytesleft);
+
 void initialize_common_converters(void);
-char *convert_string(iconv_t converter, const char *src, int length);
+char *convert_string(iconv_t converter, iconv_helper *helper,
+		     const char *src, int length);
+size_t cp932_to_utf8_helper(const char **inbuf, size_t *inbytesleft,
+			    char **outbuf, size_t *outbytesleft);
 
 gchar *wipe_string(const gchar *string);
 
-#if 1	/* to be obsoleted soon */
+#if 0	/* to be obsoleted soon */
 char *url_extract_http_server(const char *url);
 char *url_extract_http_root_pathname(const char *url);
 char *url_encode_string(const char *src);
